using Microsoft.EntityFrameworkCore;
using PetCompanion.Models;

namespace PetCompanion.Data
{
    public class ApplicationDbContext : DbContext
    {
        public ApplicationDbContext(DbContextOptions<ApplicationDbContext> options) : base(options)
        {
        }

        public DbSet<Pet> Pets { get; set; }
        public DbSet<PetStats> PetStats { get; set; }
        public DbSet<Resources> Resources { get; set; }
        public DbSet<Skill> Skills { get; set; }
        public DbSet<SkillEffect> SkillEffects { get; set; }
        public DbSet<PetSkill> PetSkills { get; set; }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            base.OnModelCreating(modelBuilder);

            // Configure DateTime properties to be stored as UTC
            modelBuilder.Entity<Pet>()
                .Property(p => p.GatherActionSince)
                .HasConversion(
                    v => v.ToUniversalTime(),
                    v => DateTime.SpecifyKind(v, DateTimeKind.Utc));

            modelBuilder.Entity<Pet>()
                .Property(p => p.BasicActionCooldown)
                .HasConversion(
                    v => v != DateTime.MinValue ? v.ToUniversalTime() : v,
                    v => v != DateTime.MinValue ? DateTime.SpecifyKind(v, DateTimeKind.Utc) : v);

            modelBuilder.Entity<PetSkill>()
                .HasOne(ps => ps.Pet)
                .WithMany(p => p.Skills)
                .HasForeignKey(ps => ps.PetId);

            modelBuilder.Entity<PetSkill>()
                .HasOne(ps => ps.Skill)
                .WithMany(s => s.PetSkills)
                .HasForeignKey(ps => ps.SkillId);

            modelBuilder.Entity<SkillEffect>()
                .HasOne(se => se.Skill)
                .WithMany(s => s.Effects)
                .HasForeignKey(se => se.SkillId);

            // Seed initial skills
            var skills = SkillsData.GetInitialSkills();
            foreach (var skill in skills)
            {
                modelBuilder.Entity<Skill>().HasData(skill);
                foreach (var effect in skill.Effects)
                {
                    modelBuilder.Entity<SkillEffect>().HasData(effect);
                }
            }
        }
    }
}
